#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Créé le Fri Nov 18 20:16:46 2022

@auteur: David ALBERTO
(www.astrolabe-science.fr)

Ce script compose un calendrier lunaire mensuel, avec l'aspect de la Lune,
et le pourcentage d'éclairement de la zone visible.
"""
# appel des modules :
import matplotlib.pyplot as plt
from matplotlib import patches
import datetime as dt
from skyfield.api import load, utc
from skyfield.framelib import ecliptic_frame
from math import ceil
import locale# noms des mois en français, sur les graduations

# Réglages à personnaliser : ----------------------------
locale.setlocale(locale.LC_ALL, 'fr_FR.UTF-8')#dates en français
plt.rcParams["font.family"] = "Roboto" # ou autre police installée
mois = 11   # numéro du mois
annee = 2022
R = 0.5 # rayon du disque lunaire
ncols = 7 # nombre de colonnes dans le mois
fig_width = 29.7 / 2.54
fig_height = 21 / 2.54

# ---------------------------------------------------
#  paramètres secondaires
nom_mois = dt.datetime(annee, mois, 1).strftime('%B')
rang_J1 = dt.datetime(annee, mois, 1).strftime('%w')# rang du 1er jour du mois dans la semaine (lundi : 1)
if rang_J1 == str(0):# dimanche
    rang_J1 = 7
else:
    rang_J1 = int(rang_J1)

ts = load.timescale()
eph = load('../skyfield-ephemerides/de421.bsp')
sun, moon, earth = eph['sun'], eph['moon'], eph['earth']

def nb_jours(mois,annee):
    """
    renvoie le nombre de jours dans le mois.
    """
    jour_un = dt.datetime(annee, mois, 1)
    if mois != 12:
        jour_mois_suivant = dt.datetime(annee, mois+1, 1)
    else:
        jour_mois_suivant = dt.datetime(annee+1, 1, 1)

    return (jour_mois_suivant - jour_un).days

NJ =nb_jours(mois, annee)
nrows = ceil((NJ+rang_J1-1)/7)

def formatage_axe(axe):
    """
    met en forme l'axe
    """
    axe.set_aspect('equal')
    axe.set_xlim(-1,1)
    axe.set_ylim(-1,1)
    axe.set_xticks([])
    axe.set_yticks([])


def efface(axe):
    """
    efface un axe inutile.
    """
    axe.axis('off')


def pourcentage(t):
    """
    Parameters
    ----------
    t : TYPE        : date Skyfield

    Renvoie l'angle de phase et le pourcentage de la partie éclairée de la Lune (entre 0 et 1)
    -------
    """

    e = earth.at(t)
    s = e.observe(sun).apparent()
    m = e.observe(moon).apparent()
    
    _, slon, _ = s.frame_latlon(ecliptic_frame)# longitude géocentrique du Soleil
    _, mlon, _ = m.frame_latlon(ecliptic_frame) # longitude géocentrique de la Lune
    phase = (mlon.degrees - slon.degrees) % 360.0

    return phase, m.fraction_illuminated(sun)

# --------------------------------------
def lune(p,phase, axe):
    """
    Parameters
    ----------
    p : TYPE float
        DESCRIPTION : pourcentage de la phase lunaire (0 à 1)
    phase : angle de phase (0 à 360°)
    axe : TYPE : axe où faire le tracé

    Trace l'apparence de la Lune (cercle, ellipse, moitié sombre)

    """
    b = R * (1-2 * p) # demi-petit axe de l'ellipse du terminateur

    limbe = patches.Circle((0,0),R,facecolor='white',edgecolor='k',zorder=0,
                           linewidth=0.5)
    axe.add_patch(limbe)
    
    if phase <= 180:
        moitie_noire = patches.Wedge((0,0), R, 90, 270, color='k',ec='None')
    else:
        moitie_noire = patches.Wedge((0,0), R, -90, 90, color='k',ec='None')

    axe.add_patch(moitie_noire)

    if phase > 90 and phase < 270:
        couleur_ellipse = 'white'
    else:
        couleur_ellipse = 'k'
    ellipse = patches.Ellipse((0,0), 2 * b, 2 * R,color=couleur_ellipse,lw=0)
    axe.add_patch(ellipse)

    # affichage du pourcentage :
    axe.text(0.98,0.98,f'{round(p*100)} %',ha='right',va='top',fontsize=14,
             c = 'orangered',
             transform=axe.transAxes)

def numero_j(J,axe):
    """
    Affiche le numéro du jour
    """
    axe.text(0.98,0.02,J,c='royalblue',fontsize=22,ha='right',fontweight='bold',
             transform=axe.transAxes)

#  fin des définitions de fonctions--------------

#  Définition de la figure et des axes :


fig, axs=plt.subplots(nrows=nrows, ncols=ncols,
                       tight_layout=True,# à retirer éventuellement, si 6 lignes
                      facecolor='skyblue')
fig.set_size_inches(fig_width, fig_height)# dimensions paysage A4 en pouces
plt.suptitle(f'{nom_mois} {annee}',fontsize=16,fontweight='bold',
             color='k',
             # y=0.95 # ajuste la position verticale du titre sur la fig.
             )
axs=axs.flatten()# permet de parcourir la grille d'axe dans une boucle

#  Boucle parcourant les jours du mois :---------------------

for J in range(1,NJ +1):
    axe = axs[J+rang_J1-2]
    date = dt.datetime(annee, mois, J,tzinfo=utc)
    jour_semaine = date.strftime('%A')
    axe.text(0.02,0.02,jour_semaine,c='royalblue',fontweight='bold',
                                 fontsize=10,
             transform=axe.transAxes)
    t = ts.from_datetime(date)
    formatage_axe(axe)
    phase, p = pourcentage(t)
    lune(p,phase, axe)
    numero_j(J, axe)

for J in range(rang_J1-1):# efface les axes inutiles au début
    axe = axs[J]
    efface(axe)
for J in range(NJ+rang_J1-1,nrows * ncols):# efface les axes inutiles à la fin
    axe = axs[J]
    efface(axe)

# ajustement des marges et des espaces entre axes, selon le nombre de lignes
if nrows == 6:
    plt.subplots_adjust(left = 0.05,
                        # wspace=0.08,
                        # hspace=0.02
                        bottom=0.05
                        )
# ---------------------------
# signature :

fig.text(0.99,0.02,'D. Alberto (www.astrolabe-science.fr)',
             fontsize=5,rotation=90,ha='right',va='bottom',
             transform=fig.transFigure)

NomFichier='Lune_' + str(annee) + '_' + str(mois)
fig.savefig(NomFichier + '.png',dpi=200)
fig.savefig(NomFichier + '.pdf')

